/**
 * @copyright COPYRIGHT © 2022, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   @file PulsarAPI.h    External definitions for the High Level User Application Programming Interface

   @brief         This module contains external definitions for the High Level User Application
                  Programming Interface DLL functions in the Pulsar320 and Pulsar320E projects.


   Written for: <br>
    Advanced Illumination, Inc.<br>
    440 State Garage Road<br>
    Rochester, VT 05767<br>
    http://advancedillumination.com<br>

   Contacts:      
    John Thrailkill<br>
    Phone:  (802) 767-3830<br>
    FAX:    (802) 767-3831<br>
    jthrailkill@advancedillumination.com<br>

   Written by:<br>
    Rob Macklin<br>
    Microprocessor Designs, Inc.<br>
    65 Longmeadow Drive<br>
    PO Box 160<br>
    Shelburne, VT 05482<br>
    http://www.updesigns.com<br>

   $Revision: 4 $
*/
#ifdef OLD_LOG
/**
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/pc_src/PulsarAPI/PulsarAPI.h $
 * 
 * 4     1/10/13 11:57a Mburl
 * "no_checks" set_api_channel_config function added for speeding up
 * settings programming. Corresponding updates for testing in
 * Pulsar320Loader.exe added.
 * 
 * 3     10/16/12 11:44a Mburl
 * brought up to date with AI code
 * 
 * 2     8/22/08 3:06p Jpiet
 * Made additions for repeat mode
 * 
 * 1     8/15/08 2:31p Jpiet
 * 
 * 17    5/15/08 4:30p Rmack
 * Changed name of init function to force error from old applications.
 * 
 * 16    5/15/08 2:38p Rmack
 * Updated for 32-bit pulse widths.
 * 
 * 15    5/02/05 8:52a Rmack
 * Dual pulse mode triggers, added new command for lighthead information,
 * separate gated continuous mode of operation.
 * 
 * 14    4/28/05 4:36p Rmack
 * Added new mode specifically for gated continuous.
 * 
 * 13    4/27/05 4:26p Rmack
 * Changed prototype for get_light_data command.
 * 
 * 12    4/27/05 8:45a Rmack
 * Added command to get lighthead data.
 * 
 * 11    2/23/05 5:17p Rmack
 * Supports channel configuration validation, and mixed mode operation.
 * 
 * 10    2/17/05 4:59p Rmack
 * Changed interface to channel configuration commands.
 * 
********************************************************************************
*******************************************************************************/
#endif


#ifndef _PULSARAPI_H
#define _PULSARAPI_H
#ifdef _WIN32
#include <wtypes.h>
#include <oleauto.h>
#endif

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the PULSARAPI_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// PULSAR_API functions CALLSPEC as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.  If a non-Microsoft compiler is used the
// PULSAR_API CALLSPEC is defined as a blank and no modifications are made to the function interfaces.
#ifdef _MSC_VER
   #define CALLSPEC __stdcall
   #ifdef PULSARAPI_EXPORTS
   #define PULSAR_API extern "C" __declspec(dllexport)
   #else
   #define PULSAR_API extern "C" __declspec(dllimport)
   #endif
#else
   #define CALLSPEC
   #define PULSAR_API extern "C"
#endif

/* Define name of USB device */
#define API_PULSAR_PRODUCT_NAME            "Ai Pulsar 320 Controller"

/* MPD Standard data type definitions */
#ifndef MPD_TYPES
#define MPD_TYPES
typedef unsigned char  UINT8_T;
typedef unsigned short UINT16_T;
typedef unsigned int   UINT32_T;
typedef char           INT8_T;
typedef short          INT16_T;
typedef int            INT32_T;
typedef unsigned char  BOOLEAN_T;
typedef float          FLOAT32_T;
/** 
 *\typedef INT16_T ERRCODE_T 
 * \brief API Error Code type -- error codes documented in separate "ecodes.ini" file 
 */
typedef INT16_T        ERRCODE_T;
#endif

/**
 * @brief Channel/output configuration struct
 * 
 */
typedef struct
{
   /// Trigger input 
   UINT8_T   trigger; 
   /// Rising edge (1 = rising, 0 = falling)
   UINT8_T   rising;
   /// Operating mode 
   /// @see API_MODE_T
   UINT8_T   mode;
   UINT8_T   reserved_byte;
   ///Number of repeat pulses per trigger (0 = 1 pulse)
   UINT32_T  repeat_cnt;
   /// Pulse delay (microseconds)
   UINT16_T  delay;
   /// Pulse current (Amps)
   FLOAT32_T current;
   /// Pulse width (microseconds)
   UINT32_T  width;
   /// Off time between repeat pulses (microseconds)
   UINT32_T  offtime;
}  API_CHAN_CONFIG_STRUCT_T;              /* 20 bytes */


/**
Define possible operating modes 
*/
enum API_MODE_T
{
   ///Pulse mode
   API_MODE_PULSED,
   ///Repeat mode, same as pulsed in 320E
   API_MODE_REPEAT,
   ///Used for Continuous mode (with 1,000,000us pulsewidth)
   API_MODE_TIMING_BYPASS,
   ///Used for test mode (10Hz internal trigger)
   API_MODE_INTERNAL_TRIGGER,
   /// Number of enum entries (do not use as mode)
   API_NUM_MODE
};

/*! Define the drive channel identifiers used with the API interface */
enum API_CHAN_T
{
   //Channel 1
   API_CHAN_1 = 1,
   //Channel 2
   API_CHAN_2,
};

#ifdef KKP
#define API_NUM_CHAN  2
#else
#define API_NUM_CHAN  2
#endif

/**
 @brief Define the trigger input identifiers used with the API interface 
*/
enum
{
   ///No trigger input
   API_TRIG_NONE,
   ///Trigger input 1
   API_TRIG_1,
   ///Trigger input 2
   API_TRIG_2,
};

#ifdef KKP
#define API_NUM_TRIG  2
#else
#define API_NUM_TRIG  2
#endif

/* Define identifiers used to handle how channel configuration data is used */
#define API_CHAN_CONFIG_DEFAULT        1
#define API_CHAN_CONFIG_ACTIVE         2

/** Define identifiers for diagnostics tests */
enum
{
   API_POWER_TEST,
   API_RAM_TEST,
   API_FLASH_TEST,
   API_EXTIO_TEST,
   API_EEPROM_TEST,
   API_EXT_DAC_TEST,
   API_SCI_TEST,
   API_NUM_TESTS
};

/* Optional diagnostics tests do not normally run on startup of the Pulsar */
/* They require additional hardware beyond the PCB Assembly itself to complete */
/* The optional SCI test requires a serial loopback connector between the external */
/* modular connector and the internal 10-pin header */
#define API_NUM_OPTIONAL_TESTS      1

/* Define bit identifiers for the information returned by the api_get_user_data command */

/* The rotary switch value is returned within a range of 0 to API_MAX_ROTARY */
#define API_MAX_ROTARY     999

/* Each bit is associated with one DIP switch position */
#define API_DIP_MODE0      0x1
#define API_DIP_MODE1      0x2
#define API_DIP_MODE2      0x4
#define API_DIP_MODE3      0x8

/* Each bit is associated with one TRIG input */
#define API_TRIG_IN1       0x1
#define API_TRIG_IN2       0x2

/* Connection state of lighthead, USB, and RS-232 */
#define API_CONN_LH_DETECT 0x1
#define API_CONN_USBDETECT 0x2
#define API_CONN_PCPRESENT 0x4
#define API_CONN_PC_READY  0x8

/* Operating state of power supplies */
#define API_SUPPLY_VIN_ON  0x1
#define API_SUPPLY_LV_ON   0x2
#define API_SUPPLY_HV_ON   0x4

/* Additional expansion general purpose inputs */
#define API_EXP_GPIN1      0x1
#define API_EXP_GPIN2      0x2
#define API_EXP_GPINOUT1   0x4 
#define API_EXP_GPINOUT2   0x8

/* Define possible lighthead types returned by api_get_light_data command */
#define API_SIGNATECH_I    0x01
#define API_SIGNATECH_II   0x02

/* Define analog input identifiers used with api_measure_voltage command */
#define API_AI_VIN_SWMON      0x6F
#define API_AI_V_HV_STRINGMON 0x8C
#define API_AI_V_LV_STRINGMON 0xFF
#define API_AI_VCH1MON        0xFF
#define API_AI_VCH2MON        0xFF
#define API_AI_VCH3MON        0xFF
#define API_AI_VCH4MON        0xFF
#define API_AI_GND1           0x74
#define API_AI_SIG1_ID1_MON   0xFF
#define API_AI_SIG1_ID2_MON   0xFF
#define API_AI_CH1_DRV_LVL    0xFF
#define API_AI_CH2_DRV_LVL    0xFF
#define API_AI_CH3_DRV_LVL    0xFF
#define API_AI_CH4_DRV_LVL    0xFF
#define API_AI_GND2           0x75
#define API_AI_10VMON         0xFF
#define API_AI_CH1CURRSENS    0xA0
#define API_AI_CH2CURRSENS    0xA1
#define API_AI_CH3CURRSENS    0xFF
#define API_AI_CH4CURRSENS    0xFF
#define API_AI_INT_TEMP       0xFF
#define API_AI_GND3           0xFF
#define API_AI_33VMON         0xFF
#define API_AI_5VMON          0x71
//New values
#define API_AI_15VMON         0x70
#define API_AI_15VSWMON       0x72
#define API_AI_USB5VMON       0x73
#define API_AI_CH1INTTEMP     0x79
#define API_AI_CH2INTTEMP     0x65


/**
   @brief Returns the current version number of this software.
   
   @return High byte is major version number, Low byte is minor version number

****************************************************************************/
PULSAR_API UINT16_T CALLSPEC api_get_dll_version(void);

/**
 * @brief Returns the full version number of this software
 * 
 * @details Each byte in the integer encodes a different part of the version number
 * <ul>
 * <li>Byte 0 (highest): MAJOR</li>
 * <li>Byte 1: MINOR</li>
 * <li>Byte 2: PATCH</li>
 * <li>Byte 3: Build (usually 0)</li>
 * </ul>
 * 
 * @return Big-endian unsigned integer encoding the version number
 */
PULSAR_API UINT32_T CALLSPEC api_get_dll_full_version(void);

/**
   @brief Returns the name of the error associated with the code

   @param[in] ecode Code to be looked up
   @param[out] nameptr Location to place name string
   @param[in] namelen Maximum length of name string


   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_error_name(ERRCODE_T ecode,char * nameptr,UINT16_T namelen);

/**
   @brief      Returns the text description of the error associated with the code

   @param ecode Code to be looked up
   @param[out] textptr Location to place text description string
   @param textlen Maximum length of text description string

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_error_text(ERRCODE_T ecode,char * textptr,UINT16_T textlen);

/**
   @brief      Initializes the connection to a specific Pulsar
   
   @warning FOR USB CONNECTIONS ONLY - SEE \ref api_connect FOR TCP/IP CONNECTIONS
   @see api_connect

   @param port Identifier for device
   @param product_name Name of USB device to open
	@param serial_num Serial number of device to open (or NULL)

   @return     Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_init(UINT8_T port, const INT8_T * product_name, const INT8_T * serial_num);

/**
   @brief      Initializes the connection to a specific Pulsar

   @see api_init for USB connections

   @param port Identifier for device
   @param product_name Name of USB device to open
   @param ip_addr IP address of device to open

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_connect(UINT8_T port, const INT8_T * product_name, const INT8_T * ip_addr);

/**
   @brief      Reads firmware version, hardware version, and serial number
               Each string can be up to 32 characters long.

   @param[in] port Identifier for device

   @param[out] fw_version Firmware version
   @param[out] hw_version Hardware version
   @param[out] serial_num Serial number

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_pulsar_version(UINT8_T port, INT8_T * fw_version, INT8_T * hw_version, INT8_T * serial_num);

/**
   @brief      Reads iip configuration settings from the device
               Each string can be up to 32 characters long.

   @param[in] port Identifier for device

   @param[out] ip_addr IP Address
   @param[out] subnet Subnet Mask
   @param[out] gateway Gateway
   @param[out] mac_addr Hardware MAC address

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_pulsar_ipaddr(UINT8_T port, INT8_T * ip_addr, INT8_T * subnet, INT8_T * gateway, INT8_T * mac_addr);

/**
   @brief Reads status information from the device

   @param port Identifier for device

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_status(UINT8_T port);

/**
   @brief  Runs one or all of the built in diagnostics

   @param port Identifier for device
   @param test_num Desired test number or FF for all

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_run_diags(UINT8_T port, UINT8_T test_num);


/**
  @brief Gets the configuration data for one drive channel

   @param port Identifier for device
   @param chan Identifier for drive channel
   @param type Identifier for which data to get
               
   @param[out] config_data Active channel configuration data

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_channel_config(UINT8_T port, UINT8_T chan, UINT8_T type,
                                               API_CHAN_CONFIG_STRUCT_T *  config_data);

/**
   @brief Sets the configuration data for one drive channel

   @param port Identifier for device
   @param chan Identifier for drive channel
   @param type Identifier for which data to get
   @param  API_CHAN_CONFIG_STRUCT_T *  config_data
                                       New channel configuration data

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_set_channel_config(UINT8_T port, UINT8_T chan, UINT8_T type,
                                               API_CHAN_CONFIG_STRUCT_T *  config_data);


/**
   @brief Sets the configuration data for one drive channel.

   @warning Unlike \ref api_set_channel_config API call, the "nochecks"
			   function does not verify the settings. This could result it
			   improper channel settings or configuration that could
			   break hardware or blow flash bulbs.

   @param port Identifier for device
   @param chan Identifier for drive channel
   @param type Identifier for which data to get
   @param  API_CHAN_CONFIG_STRUCT_T *  config_data
                                       New channel configuration data

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_set_channel_config_nochecks(UINT8_T port, UINT8_T chan, UINT8_T type,
                                               API_CHAN_CONFIG_STRUCT_T *  config_data);


/**
   @brief  Generates a software trigger on the selected trigger channel
   
   @param port Identifier for device
   @param trig Trigger to activate
   
   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_trigger(UINT8_T port, UINT8_T trig);


/**
   @brief   Calculates the expected LED junction temperature based on the
            information stored in the lighthead data structure and the width
            and current information passed.

   @param port Identifier for device
   @param ch Channel to compute LED temperature
   @param width Pulse width in microseconds
   @param current Pulse current in amps
   
   @param[out] temp Location to store calculated temperature

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_calc_led_temp(UINT8_T port, UINT8_T ch, 
                                       UINT32_T width, FLOAT32_T current, FLOAT32_T *temp);

/**
   @brief   Calculates the maximum pulse width for the selected channel based 
            on the information stored in the lighthead data structure and the 
            current information passed.

   @param port Identifier for device
   @param ch Channel to compute LED temperature
   @param current Pulse current in amps
   
   @param[out] width Maximum pulse width in microseconds

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_calc_max_width(UINT8_T port, UINT8_T ch, 
                                        FLOAT32_T current, UINT32_T * width);

/**
   @brief   Calculates the maximum pulse current for the selected channel based 
            on the information stored in the lighthead data structure and the 
            width information passed.

   @param port Identifier for device
   @param ch Channel to compute LED temperature
   @param width Pulse width in microseconds
   
   @param[out] current Maximum pulse current in amps

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_calc_max_current(UINT8_T port, UINT8_T ch, 
                                          UINT32_T width, FLOAT32_T *current);

/**
   @brief   Calculates the required supply voltage for a given width and current.

   @param port Identifier for device
   @param ch Channel to compute required supply voltage
   @param width Pulse width in microseconds
   @param current Pulse current in amps
   
   @param[out] voltage Location to store calculated voltage

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_calc_supply_voltage(UINT8_T port, UINT8_T ch, 
                                                    UINT32_T width, FLOAT32_T current, FLOAT32_T *voltage);

/**
   @brief   Calculates the maximum frequency allowed for a given width and current.
            This function computes maximum frequency based on the maximum power of
            the power supply and also from the LED cooling equations.  The lower
            value is returned.

   @param port Identifier for device
   @param ch Channel to compute maximum frequency
   @param width Pulse width in microseconds
   @param current Pulse current in amps
   
   @param[out] freq        Location to store calculated frequency

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_calc_max_freq(UINT8_T port, UINT8_T ch, 
                                       UINT32_T width, FLOAT32_T current, UINT16_T *freq);

/**
   @brief   Measures the drive current of the selected drive channel.  This 
            function can be used in either pulse or continuous modes.

   @param port Identifier for device
   @param ch Channel to measure pulse current
   
   @param[out] current     Location to store measured pulse current

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_measure_drive_current(UINT8_T port, UINT8_T ch, FLOAT32_T *current);

/**
   @brief   Measures the voltage of one of the signals monitored by the Pulsar.
   
   @param port Identifier for device
   @param id          Identifier for desired voltage to measure
   
   @param[out] voltage     Location to store measured voltage

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_measure_voltage(UINT8_T port, UINT8_T id, FLOAT32_T *voltage);

/**
   @brief   Gathers various information about the operating state of the Pulsar.

   @param port Identifier for device
   
   @param[out] rotary Value on rotary switches
   @param[out] dip Value on DIP switches
   @param[out] trigger State of trigger inputs
   @param[out] connect Communication connection status
   @param[out] supply Power supply status
   @param[out] expansion Expansion status

   @return Zero if successful, error code otherwise

   @note   A NULL can be passed for any data values not needed
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_user_data(UINT8_T port,UINT16_T *rotary,UINT8_T *dip,UINT8_T *trigger,
                                       UINT8_T *connect,UINT8_T *supply,UINT8_T *expansion);

/**
   @brief   Gathers information regarding the lighthead type.

   @param port Identifier for device
   
   @param[out] sig_model 1 - Signatech I, 2 - Signatech II, 0 - Unknown
   @param[out] sig1_id Set for Signatech I lightheads
   @param[out] sig2_pn Set to part number string for Signatech II
   @param[out] sig2_sn Set to serial number of Signatech 2 lighthead

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_light_data(UINT8_T port, UINT8_T *sig_model, UINT8_T *sig1_id, char *sig2_pn, char *sig2_sn);

/**
 * @brief Get Signatech II lighthead part and serial number. 
 * 
 * @param port Identifier for device
 * @param channel Output channel to read (only valid for Pulsar 320E)
 * @param[out] sig2_pn Part number of Signatech 2 lighthead
 * @return Zero if successful, error code otherwise. 
 */
PULSAR_API ERRCODE_T CALLSPEC api_get_light_data2(UINT8_T port, UINT8_T channel, char *sig2_pn);

/**
   @brief   Saves a block of Ai defined data to EEPROM.
   
   @param port Identifier for device
   @param data Pointer to data block to be written
   @param len Length of data block to be written

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_set_stored_data(UINT8_T port, UINT8_T *data, UINT8_T len);

/**
   @brief   Reads a block of Ai defined data from EEPROM.
   
   @param port Identifier for device            
   @param len Length of data block to read
   
   @param[out] data Pointer to data block to be written

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_get_stored_data(UINT8_T port, UINT8_T *data, UINT8_T len);



/**
   @brief      Shuts down the connection to a specific Pulsar

   @param port Communication port to use

   @return Zero if successful, error code otherwise
*/
PULSAR_API ERRCODE_T CALLSPEC api_shutdown(UINT8_T port);

/**
 * @brief Set hardware configuration
 * 
 * @param port Communication port to use
 * @param[in] slew_rate Slew rate (array, one for each channel)
 * @param[in] diff_trigger Differential trigger (0 or 1) (array, one for each channel)
 * @param[in] limit48v Limit output to 48v
 * @param[in] disable_prot Disable protection
 * @return Zero if successful, error code otherwise
 */
PULSAR_API ERRCODE_T CALLSPEC api_set_configuration(UINT8_T port, UINT8_T* slew_rate, UINT8_T* diff_trigger, UINT8_T limit48v, UINT8_T disable_prot);

/**
 * @brief Get hardware configuration
 * 
 * @param port Communication port to use
 * @param[out] slew_rate Slew rate (array, one for each channel)
 * @param[out] diff_trigger Differential trigger (0 or 1) (array, one for each channel)
 * @param[out] limit48v Limit output to 48v
 * @param[out] disable_prot Disable protection
 * @return Zero if successful, error code otherwise
 */
PULSAR_API ERRCODE_T CALLSPEC api_get_configuration(UINT8_T port, UINT8_T* pslew_rate, UINT8_T* pdiff_trigger, UINT8_T* limit48v, UINT8_T* disable_prot);

/**
 * @brief Set Pulsar IP address
 * 
 * @param port Communication port to use
 * @param[in] ip_addr IP address
 * @param[in] subnet Subnet mask
 * @param[in] gateway Gateway address
 * @return Zero if successful, error code otherwise 
 */
PULSAR_API ERRCODE_T CALLSPEC  api_set_pulsar_ipaddr(UINT8_T port, const char* ip_addr, const char* subnet, const char* gateway);

/**
 * @brief Get Pulsar IP address
 * 
 * @param port Communication port to use
 * @param[out] ip_addr IP address
 * @param[out] subnet Subnet mask
 * @param[out] gateway Gateway address
 * @param[out] mac_addr MAC address
 * @return Zero if successful, error code otherwise 
 */
PULSAR_API ERRCODE_T CALLSPEC  api_get_pulsar_ipaddr(UINT8_T port, char* ip_addr, char* subnet, char* gateway, char* mac_addr);

/**
 * @brief Reads the oldest posted error
 * 
 * @param port Communication port to use
 * @param[out] err The oldest posted error in the device
 * @param[out] totalErrors The number of errors remaining to read
 * @return PULSAR_API 
 */
PULSAR_API ERRCODE_T CALLSPEC api_get_errors(UINT8_T port, INT16_T* err, UINT8_T* totalErrors);

/**
 * @brief Gets the type of device on the given port
 * 
 * @param port Communication port of the device
 * @return Zero if none connected, 1 for Pulsar 320, 2 for Pulsar 320E 
 */
PULSAR_API int api_connected_device(UINT8_T port);

// @private
PULSAR_API ERRCODE_T CALLSPEC api_get_mac(UINT8_T port, UINT8_T* mac);
// @private
PULSAR_API ERRCODE_T CALLSPEC api_set_mac(UINT8_T port, UINT8_T* mac);
// @private
PULSAR_API ERRCODE_T CALLSPEC api_get_manufacturing_mac(UINT8_T port, UINT8_T* mac);

/**
 * @brief Find pulsar devices on all attached network adapters.
 * 
 * @param addrs Array of UINT32_Ts to fill with found IP addresses (in network byte order).
 * @param max_addrs Maximum number of addresses to find (size of the array)
 * @param num_found Actual number of addresses found
 * @return Zero if successful, error code otherwise
 */
PULSAR_API ERRCODE_T CALLSPEC api_get_pulsar_addrs(UINT32_T* addrs, UINT16_T max_addrs, UINT16_T* num_found);

#if defined(_WIN32) && defined(USE_SAFEARRAY)
long __declspec (dllexport) __stdcall AddLongs_SafeArray(
      SAFEARRAY **psaArray, // the array to use
      long *plSum); // returns the sum of all elements of the array
#endif

#endif /* _PULARAPI_H */
